
#include "mbc/mbc1.h"

/* Memory bank pointers */
static unsigned char *rom0;
static unsigned char *rom1;
static unsigned char *exram;

static unsigned char *rombanks;
static unsigned char *rambanks;

static int enable_ram=0;
static int bank_mode=0;
static int ram_size=0;

static unsigned char ram_bank_num=0;
static unsigned char rom_bank_num=0x01;

static void mbc1_reset();

static unsigned char dummy[0x2000]={0};

int mbc1_load(cart_info *info)
{
  rombanks = info->rombanks;
  rambanks = info->rambanks;

  rom0 = rombanks;
  rom1 = &(rombanks[0x4000]);
  
  if (rambanks) {
    ram_size = info->ramsize;
    exram = rambanks;
  }
  else {
    exram = dummy;
  }

  set_rom0_source(rom0);
  set_rom1_source(rom1);
  set_exram_source(exram);

  set_rom0_handlers(mbc1_rom0_read, mbc1_rom0_write);
  set_rom1_handlers(mbc1_rom1_read, mbc1_rom1_write);
  set_exram_handlers(mbc1_exram_read, mbc1_exram_write);
}

unsigned char mbc1_rom0_read(unsigned short addr)
{
  return rom0[addr];
}

void mbc1_rom0_write(unsigned short addr, unsigned char value)
{
  unsigned char bank_num;

  /* RAM Enable */
  if (addr <= 0x1FFF) {
    if ((value & 0x0F) == 0x0A) {
      enable_ram = ( enable_ram ? 0 : 1 );
    }
  }
  /* ROM Bank number */
  else if ( (addr >= 0x2000) && (addr <= 0x3FFF) ) {
    if (value == 0) value=1;
    
    /* ROM banking mode */
    if (bank_mode == 0) {
      rom_bank_num = (ram_bank_num << 6) | value;
    }
    /* RAM Banking mode */
    else {
      rom_bank_num = value;
    }

    rom1 = &(rombanks[rom_bank_num*0x4000]);
    set_rom1_source(rom1);


  }

}

unsigned char mbc1_rom1_read(unsigned short addr)
{
  return rom1[addr];
}

void mbc1_rom1_write(unsigned short addr, unsigned char value)
{
  /* RAM Bank number OR High 2 bits of ROM Bank number */
  if (addr <= 0x1FFF) {
    ram_bank_num = value & 3;

    if (!rambanks) return;
    if (ram_size != 0x8000) return;

    /* ROM banking mode */
    if (bank_mode == 0) {
      rom_bank_num = (ram_bank_num << 6) | value;
      rom1 = &(rombanks[rom_bank_num*0x4000]);
      set_rom1_source(rom1);

    }
    /* RAM banking mode */
    else {
      exram = &(rambanks[ram_bank_num*0x1000]);
      set_exram_source(exram);

    }

  }
  /* Banking mode */
  else if ( (addr >= 2000) && (addr <= 0x3FFF) ) {
    bank_mode = value;
  }
}

unsigned char mbc1_exram_read(unsigned short addr)
{
  if (!enable_ram) return 0xFF;
  if (ram_size == 0) return 0xFF;

  if (ram_size <= 0x800) {
    return ( (addr<ram_size) ? exram[addr] : 0xFF );
  }
  else if (ram_size <= 0x1000) {
    return ( (addr<ram_size) ? exram[addr] : 0xFF );
  }
  else {
    return exram[addr];
  }

}

void mbc1_exram_write(unsigned short addr, unsigned char value)
{
  if (!enable_ram) return;
  if (ram_size == 0) return;

  if (ram_size <= 0x800) {
    if (addr < ram_size) exram[addr] = value;
  }
  else if (ram_size <= 0x1000) {
    if (addr < ram_size) exram[addr] = value;
  }
  else {
    exram[addr]=value;
  }

}

void mbc1_reset()
{
  rom0 = rombanks;
  rom1 = &(rombanks[0x4000]);
  if (rambanks) 
    exram = rambanks;
  else
    exram = dummy;

  set_rom0_source(rom0);
  set_rom1_source(rom1);
  set_exram_source(exram);

}
