
#include <stdio.h>
#include "mbc/mbc2.h"

/* Memory banks */
static unsigned char *rom0;
static unsigned char *rom1;
static unsigned char exram[0x2000] = {0};

static unsigned char *banks;

/* set when EXRAM is enabled */
static int enable_ram=0;

/* reset function */
static void mbc2_reset();

int mbc2_load(cart_info *this, const char *fname)
{
  mbc_reset = mbc2_reset;

  banks = this->banks;

  rom0 = &(banks[0]);
  rom1 = &(banks[0x4000]);
 
  set_rom0_source(rom0);
  set_rom1_source(rom1);
  set_exram_source(exram);

  set_rom0_handlers(mbc2_rom0_read, mbc2_rom0_write);
  set_rom1_handlers(mbc2_rom1_read, mbc2_rom1_write);
  set_exram_handlers(mbc2_exram_read, mbc2_exram_write);

}

/* Memory access handlers */
unsigned char mbc2_rom0_read(unsigned short addr)
{
  return rom0[addr];
}

void mbc2_rom0_write(unsigned short addr, unsigned char value)
{
  unsigned char bank_num;

  /* RAM enable */
  if (addr <= 0x1FFF) {
    if ( ((addr&0xF0)&0x10) == 0 ) {
      enable_ram = (enable_ram ? 0 : 1);
    }
  }
  /* ROM Bank # */
  else if ( (addr >= 0x2000) && (addr <= 0x3FFF) ) {
    if ( ((addr&0xF0)&0x10) == 0 ) {
      bank_num = value;
      printf("%s: switching ROM1 to bank #%.2x\n", __FILE__, bank_num);
      rom1 = &(banks[bank_num*0x4000]);
      set_rom1_source(rom1);
    }
  }

}

unsigned char mbc2_rom1_read(unsigned short addr)
{
  return rom1[addr];
}

void mbc2_rom1_write(unsigned short addr, unsigned char value)
{
  /* WRITE ONLY */
}

unsigned char mbc2_exram_read(unsigned short addr)
{
  if (enable_ram) {
    if (addr <= 0x1FF) {
      return exram[addr];
    }
    else {
      return 0xFF;
    }
  }
  else {
    return 0xFF;
  }
}

void mbc2_exram_write(unsigned short addr, unsigned char value)
{
  if (enable_ram) {
    if (addr <= 0x1FF) {
      exram[addr] = value;
    }
  }
}

static void mbc2_reset()
{
  rom0 = &(banks[0]);
  rom1 = &(banks[0x4000]);

  set_rom0_source(rom0);
  set_rom1_source(rom1);
}
